#
# spec file for package dehydrated
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define _challengedir %{_localstatedir}/lib/acme-challenge
%define _user         dehydrated
%define _home         %{_sysconfdir}/dehydrated
%define _postrunhooks %{_home}/postrun-hooks.d

%if 0%{?suse_version} > 1230
%define  _lock_dir /run/dehydrated
%bcond_without systemd
%else
%define  _lock_dir %{_localstatedir}/run/dehydrated
%bcond_with    systemd
%endif

%if 0%{?is_opensuse} || %{defined fedora}
%bcond_without nginx
%bcond_without lighttpd
%else
%bcond_with nginx
%bcond_with lighttpd
%endif
%{!?_tmpfilesdir: %global _tmpfilesdir %{_prefix}/lib/tmpfiles.d }
# See also http://en.opensuse.org/openSUSE:Specfile_guidelines

%if 0%{?suse_version}
%define _apache apache2
%else
%define _apache httpd
%endif

Name:           dehydrated
Version:        0.6.2
Release:        lp150.1.1
Summary:        A client for signing certificates with an ACME server
License:        MIT
Group:          Productivity/Networking/Security
Url:            https://github.com/lukas2511/dehydrated
Source0:        %{name}-%{version}.tar.gz
Source1:        acme-challenge.conf.apache.in
Source2:        acme-challenge.conf.nginx.in
Source3:        acme-challenge.conf.lighttpd.in
Source4:        dehydrated.cron.in
Source5:        dehydrated.tmpfiles.d
Source6:        dehydrated.service.in
Source7:        dehydrated.timer
Source9:        README.SUSE
Source10:       README.Fedora
Source11:       README.hooks
Source12:       %{name}-%{version}.tar.gz.asc
Source13:       %{name}.keyring
BuildRequires:  %{_apache}
Requires:       coreutils
Requires:       curl
Requires:       openssl
Requires:       sudo
Requires(pre):  %{_bindir}/getent
Requires(pre):  %{_sbindir}/groupadd
Requires(pre):  %{_sbindir}/useradd
Obsoletes:      letsencrypt.sh < %{version}
Provides:       letsencrypt.sh = %{version}
%if %{with lighttpd}
BuildRequires:  lighttpd
%endif
%if %{with nginx}
BuildRequires:  nginx
%endif
%if %{defined fedora}
BuildRequires:  generic-logos
BuildRequires:  generic-logos-httpd
%endif
# openSUSE >= 12.3 has shadow, pwdutils is provided but obsoleted.
%if 0%{?suse_version} >= 1230
BuildRequires:  shadow
%endif
%if %{with systemd}
BuildRequires:  pkgconfig(systemd)
%{?systemd_requires}
%else #with_systemd
%if 0%{?suse_version}
Requires:       cron
%endif
%endif #with_systemd
%if 0%{?suse_version}
Recommends:     dehydrated-acmeresponder
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildArch:      noarch

%description
This is a client for signing certificates with an ACME server
(currently only provided by letsencrypt) implemented as a relatively
simple bash-script.

It uses the openssl utility for everything related to actually
handling keys and certificates, so you need to have that installed.

Other dependencies are: curl, sed, grep, mktemp (all found on almost
any system, curl being the only exception)

Current features:

* Signing of a list of domains
* Signing of a CSR
* Renewal if a certificate is about to expire or SAN (subdomains) changed
* Certificate revocation

%package %{_apache}
Summary:        Apache Integration for dehydrated
Group:          Productivity/Networking/Security
Requires:       %{_apache}
Requires:       %{name}
Obsoletes:      letsencrypt.sh-%{_apache} < %{version}
Provides:       letsencrypt.sh-%{_apache} = %{version}
%if ! 0%{?suse_version}
Requires:       mod_ssl
%endif

%description %{_apache}
This adds a configuration file for dehydrated's acme-challenge to Apache.

%if %{with nginx}
%package nginx
Summary:        Nginx Integration for dehydrated
Group:          Productivity/Networking/Security
Requires:       %{name}
Requires:       nginx
Obsoletes:      letsencrypt.sh-nginx < %{version}
Provides:       letsencrypt.sh-nginx = %{version}

%description nginx
This adds a configuration file for dehydrated's acme-challenge to nginx.
%endif #with nginx

%if %{with lighttpd}
%package lighttpd
Summary:        Lighttpd Integration for dehydrated
Group:          Productivity/Networking/Security
Requires:       %{name}
Requires:       lighttpd

%description lighttpd
This adds a configuration file for dehydrated's acme-challenge to lighttpd.
%endif #with lighttpd

%pre
getent group %{_user} >/dev/null || %{_sbindir}/groupadd -r %{_user}
getent passwd %{_user} >/dev/null || %{_sbindir}/useradd -g %{_user} \
	-s /bin/false -r -c "%{_user}" -d %{_home} %{_user}
if [ -d %{_sysconfdir}/letsencrypt.sh ]; then mv %{_sysconfdir}/letsencrypt.sh %{_sysconfdir}/dehydrated; chown -R %{_user} %{_sysconfdir}/dehydrated; fi
if [ -e %{_sysconfdir}/dehydrated/config.sh ]; then mv %{_sysconfdir}/dehydrated/config.sh %{_sysconfdir}/dehydrated/config; fi

%if %{with systemd}
%service_add_pre dehydrated.service dehydrated.timer

%post
systemd-tmpfiles --create %{_tmpfilesdir}/%{name}.conf ||:
%service_add_post dehydrated.service dehydrated.timer

%preun
%service_del_preun dehydrated.service dehydrated.timer

%postun
%service_del_postun dehydrated.service dehydrated.timer
%endif

%prep
%setup -q
cp %{SOURCE9} .
cp %{SOURCE10} .

%build

%install
# sensitive keys
mkdir -p %{buildroot}%{_home}/{accounts,certs,chains}
mkdir -p %{buildroot}%{_sbindir}
mkdir -p %{buildroot}%{_mandir}/man1
mkdir -p %{buildroot}%{_home}/config.d
mkdir -p %{buildroot}%{_postrunhooks}

cat docs/man/dehydrated.1 | gzip > %{buildroot}%{_mandir}/man1/dehydrated.1.gz

# Silence E: env-script-interpreter
find \( -name \*.sh -o -name dehydrated \) -exec sed -i "s,#!/usr/bin/env bash,#!$(command -v bash),g" {} \;

sed -i "s,#WELLKNOWN=.*,WELLKNOWN=%{_challengedir},g" docs/examples/config
install -m 0644 docs/examples/* %{buildroot}%{_home}
install -m 0644 %{SOURCE11} %{buildroot}%{_postrunhooks}
install -m 0755 -d %{buildroot}%{_bindir}
install -m 0755 dehydrated %{buildroot}%{_bindir}
install -m 0755 -d %{buildroot}%{_challengedir}

install -m 0755 -d %{buildroot}%{_sysconfdir}/%{_apache}/conf.d
sed "s,@CHALLENGEDIR@,%{_challengedir},g" %{SOURCE1} > acme-challenge.conf
install -m 0644 acme-challenge.conf %{buildroot}%{_sysconfdir}/%{_apache}/conf.d

%if %{with nginx}
install -m 0755 -d %{buildroot}%{_sysconfdir}/nginx
sed "s,@CHALLENGEDIR@,%{_challengedir},g" %{SOURCE2} > acme-challenge
install -m 0644 acme-challenge %{buildroot}%{_sysconfdir}/nginx
%endif #with nginx

%if %{with lighttpd}
install -m 0755 -d %{buildroot}%{_sysconfdir}/lighttpd/conf.d
sed "s,@CHALLENGEDIR@,%{_challengedir},g" %{SOURCE3} > acme-challenge
install -m 0644 acme-challenge %{buildroot}%{_sysconfdir}/lighttpd/conf.d
%endif #with lighttpd

%if %{with systemd}
install -D    -m 0644 %{SOURCE5} %{buildroot}%{_tmpfilesdir}/%{name}.conf
# Use timer
sed "s,@POSTRUNHOOKS_DIR@,%{_postrunhooks},g" %{SOURCE6} > dehydrated.service
install -D -m 644 dehydrated.service  %{buildroot}%{_unitdir}/dehydrated.service
install -D -m 644 %{SOURCE7}          %{buildroot}%{_unitdir}/dehydrated.timer
%if 0%{?suse_version}
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rcdehydrated
%endif
%else #with systemd
install -D -d -m 0750 %{buildroot}%{_lock_dir}
# Use cron
install -m 0755 -d %{buildroot}%{_sysconfdir}/cron.d
sed "s,@POSTRUNHOOKS_DIR@,%{_postrunhooks},g" %{SOURCE4} > dehydrated.cron
install -m 0644 dehydrated.cron %{buildroot}%{_sysconfdir}/cron.d/dehydrated
%endif #with systemd

# Adjust config file
perl -p -i -e 's|#LOCKFILE="\$\{BASEDIR\}/lock"|LOCKFILE="%{_lock_dir}/lock"|' %{buildroot}%{_home}/config
perl -p -i -e 's|#CONFIG_D=|CONFIG_D="%{_home}/config.d"|' %{buildroot}%{_home}/config
perl -p -i -e 's|#DEHYDRATED_USER=|DEHYDRATED_USER="%{_user}"|' %{buildroot}%{_home}/config
perl -p -i -e 's|#DEHYDRATED_GROUP=|DEHYDRATED_GROUP="%{_user}"|' %{buildroot}%{_home}/config

diff -urN docs/examples/config %{buildroot}%{_home}/config ||:

%files
%defattr(-,root,root)
%attr(750,root,%{_user}) %dir %{_sysconfdir}/dehydrated
%attr(700,%{_user},%{_user}) %dir %{_sysconfdir}/dehydrated/accounts
%attr(700,%{_user},%{_user}) %dir %{_sysconfdir}/dehydrated/certs
%attr(700,%{_user},%{_user}) %dir %{_sysconfdir}/dehydrated/chains
%config(noreplace) %attr(640,root,%{_user}) %{_sysconfdir}/dehydrated/config
%config(noreplace) %attr(750,root,%{_user}) %{_sysconfdir}/dehydrated/config.d
%config(noreplace) %attr(640,root,%{_user}) %{_sysconfdir}/dehydrated/domains.txt
%config(noreplace) %attr(750,root,%{_user}) %{_sysconfdir}/dehydrated/hook.sh
%dir %attr(750,root,%{_user}) %{_postrunhooks}
%config(noreplace) %attr(640,root,%{_user}) %{_postrunhooks}/README.hooks
%{_bindir}/dehydrated
%attr(-,%{_user},root) %dir %{_localstatedir}/lib/acme-challenge
%{_mandir}/man1/*
%doc LICENSE README.md docs/*.md docs/*.jpg
%doc README.SUSE
%if %{defined redhat}
%doc README.Fedora
%endif
%if %{with systemd}
%{_tmpfilesdir}/%{name}.conf
%{_unitdir}/dehydrated.service
%{_unitdir}/dehydrated.timer
%{_sbindir}/rcdehydrated
%ghost %attr(700,%{_user},%{_user}) %dir %{_lock_dir}
%else
%config %{_sysconfdir}/cron.d/dehydrated
%attr(700,%{_user},%{_user}) %dir %{_lock_dir}
%endif

%files %{_apache}
%defattr(-,root,root)
%config %{_sysconfdir}/%{_apache}/conf.d/acme-challenge.conf

%if %{with nginx}
%files nginx
%defattr(-,root,root)
%config %attr(640,root,nginx) %{_sysconfdir}/nginx/acme-challenge
%endif #with nginx

%if %{with lighttpd}
%files lighttpd
%defattr(-,root,root)
%config %attr(640,root,lighttpd) %{_sysconfdir}/lighttpd/conf.d/acme-challenge
%endif #with lighttpd

%changelog
* Fri Apr 27 2018 daniel.molkentin@suse.com
- Update to dehydrated 0.6.2
  * removes 0001-fixed-CA-url-in-example-config.patch
  * removes 0002-don-t-walk-certificate-chain-for-ACMEv2-certificate-.patch
  Added
  * New deploy_ocsp hook
  * Allow account registration with custom key
  Changed
  * Don't walk certificate chain for ACMEv2 (certificate contains chain by default)
  * Improved documentation on wildcards
  Fixes
  * Added workaround for compatibility with filesystem ACLs
  * Close unwanted external file-descriptors
  * Fixed JSON parsing on force-renewal (bsc#1091216)
  * Fixed cleanup of challenge files/dns-entries on validation errors
  * A few more minor fixes
* Thu Mar 15 2018 daniel.molkentin@suse.com
- Don't add intermediate certificates twice when using ACMEv2 (bsc#1085305)
  * Adds 0002-don-t-walk-certificate-chain-for-ACMEv2-certificate-.patch
* Wed Mar 14 2018 daniel.molkentin@suse.com
- Fix issues introduced by 0.6.1 (bsc#1085305)
  * bring back man page
  * reflect new endpoint in (commented out) config file section
    (adds 0001-fixed-CA-url-in-example-config.patch, backported
    from upstream's master branch)
* Tue Mar 13 2018 daniel.molkentin@suse.com
- Updated dehydrated to 0.6.1 (bsc#1084854)
  * Use new ACME v2 endpoint by default
* Mon Mar 12 2018 daniel.molkentin@suse.com
- Updated dehydrated to 0.6.0 (bsc#1084854)
  Changed
  * Challenge validation loop has been modified to loop over authorization identifiers instead of altnames (ACMEv2 + wildcard support)
  * Removed LICENSE parameter from config (terms of service is now acquired directly from the CA directory)
  Added
  * Support for ACME v02 (including wildcard certificates!)
  * New hook: generate_csr (see example hook script for more information)
  * Calling random hook on startup to make it clear to hook script authors that unknown hooks should just be ignored...
* Mon Jan 15 2018 daniel.molkentin@suse.com
- Remove redundant noarch entries. They cause an error in RPM 4.14.
* Mon Jan 15 2018 daniel.molkentin@suse.com
- Updated dehydrated to 0.5.0
  This removes the following patches and files, which are now part of the
  upstream package:
  * 0001-Add-optional-user-and-group-configuration.patch
  * 0002-use-nullglob-disable-warning-on-empty-CONFIG_D-direc.patch
  * dehydrated.1: the man page has been adopted by upstream
  Starting with this version, upstream introduced signed releases, which
  is now being used for source validation.
  Upstream changes:
  Changed
  * Certificate chain is now cached (CHAINCACHE)
  * OpenSSL binary path is now configurable (OPENSSL)
  * Cleanup now also moves revoked certificates
  Added
  * New feature for updating contact information (--account)
  * Allow automatic cleanup on exit (AUTO_CLEANUP)
  * Initial support for fetching OCSP status to be used for OCSP stapling
    (OCSP_FETCH)
  * Certificates can now have aliases to create multiple certificates with
    identical set of domains (see --alias and domains.txt documentation)
  * Allow dehydrated to run as specified user (/group). This was already
    available previously as a patch to this package.
* Fri Oct 20 2017 mrueckert@suse.de
- revert accidental change to the service file
* Fri Oct 20 2017 mrueckert@suse.de
- actually try to find the real path to bash and don't hardcode
  /usr/bin/bash
* Thu Oct 19 2017 daniel.molkentin@suse.com
- Use /usr/bin/bash directly, rather than via env
* Wed Oct 18 2017 daniel.molkentin@suse.com
- Use sudo instead of su to allow for argument handling, also
  works in all cases when no login shell is assigned to the
  dehydrated user
  * updates 0001-Add-optional-user-and-group-configuration.patch
* Tue Oct 17 2017 daniel.molkentin@suse.com
- Commands in service files need some escaping after all. Fix ExecStartPost.
* Mon Oct 16 2017 daniel.molkentin@suse.com
- In the timer service, execute root post run hooks in ExecStartPost
* Mon Oct 16 2017 daniel.molkentin@suse.com
- Fix run of root hooks
- Simplify root hook execution, this is also more robust
* Thu Oct  5 2017 daniel.molkentin@suse.com
- Remove unused hooks directory
- Introduced a directory for custom post-run hooks executed as root,
  see README.SUSE for details. (not to be confused with the native hooks
  run as dehyrated user)
* Fri Sep 29 2017 daniel.molkentin@suse.com
- Clarify necessity of enabling dehydrated.timer in README.SUSE
- Submit to SLE15 as per fate#323377
- Add optional post run hook directory, executed by cron/systemd
  after dehydrated --cron has run
- Remove hook directory intended for packaging other native hooks.
  Will be approach differently
* Wed Sep 27 2017 daniel.molkentin@suse.com
- No longer require nginx or lighttpd for SLE
- Never go as far as to require acmeresponder, it might not be available
- Drop -update from dehydrated-update.{timer,socket} for consistency
- Add distro specific README.SUSE / README.Fedora
- Ran spec-cleaner
* Fri Sep 22 2017 daniel.molkentin@suse.com
- Add man page
- Ensure dehydrated is always run as designated user
  * adds 0001-Add-optional-user-and-group-configuration.patch
- Introduce config.d directory for user configuration
- Avoid warning about empty config.d directory
  * adds 0002-use-nullglob-disable-warning-on-empty-CONFIG_D-direc.patch
- Fix sed warning about unescaped curly braces in regex
* Tue Sep 19 2017 daniel.molkentin@suse.com
- Swap statements in post: installing services requires tmp.d
* Tue Sep 19 2017 daniel.molkentin@suse.com
- (Weak) dependency on dehydrated-acmeresponder.
* Thu Sep 14 2017 daniel.molkentin@suse.com
- systemd update service: ConditionPathExists goes into [Unit] section
* Wed Sep 13 2017 daniel.molkentin@suse.com
- Use timer instead of cron for systemd-enabled distros
  Note: Timer must be explicitly enabled!
* Tue Feb 21 2017 daniel.molkentin@suse.com
- Drop the (undocumented) dependeny for mod_headers
* Sat Feb 18 2017 daniel@molkentin.de
- Unify configuration file source names
* Sat Feb 18 2017 daniel@molkentin.de
- Bump to 0.4.0
* Thu Feb  2 2017 daniel.molkentin@suse.com
- More dependency fixes
* Thu Feb  2 2017 daniel.molkentin@suse.com
- Make nginx and lighttpd packages into features
  Default-disable them on distros where we cannot provide a dependency.
* Thu Feb  2 2017 daniel.molkentin@suse.com
- Fix build on Fedora
* Thu Feb  2 2017 mrueckert@suse.de
- make permissions of the lighty and nginx config files tighter
* Thu Feb  2 2017 mrueckert@suse.de
- only own the configuration files and not the whole directory tree
  - add BR for nginx, lighttpd, apache2 to handle directory
    ownership
* Thu Jan 12 2017 mrueckert@suse.de
- with making the permissions more tight ... dehydrated can not
  write its lock file anymore to /etc/dehydrated. To fix this we
  now create /var/run/dehydrated (sysvinit) or /run/dehydrated
  (systemd) and point the lock file in the default config to that
  directory.
  Please adapt your local config files accordingly.
* Thu Jan 12 2017 mrueckert@suse.de
- change permissions of /etc/dehydrated to:
  root:dehydrated u=rwx,g=rx,o=
- create the subdirs that dehydrated would create later anyway:
  /etc/dehydrated/accounts
  /etc/dehydrated/certs
  dehydrated::dehydrated u=rwx,go=
- tighten up permissions on
  /etc/dehydrated/config
  /etc/dehydrated/domain.txt
  root:root u=rw,go=r -> root:dehydrated u=rw,g=r,o=
  /etc/dehydrated/hook.sh
  root:root u=rw,go=r -> root:dehydrated u=rwx,g=rx,o=
* Wed Nov 23 2016 daniel@molkentin.de
- Add lighttpd configuration via dehydrated-lighttpd
* Mon Nov 14 2016 jengelh@inai.de
- Test for user/group before adding them and don't suppress errors
* Thu Nov 10 2016 daniel@molkentin.de
- Fix MIN HOUR order in crontab (boo#1009452)
* Tue Sep 13 2016 danimo@owncloud.com
- Bump to v0.3.1
- Rename to dehydrated
* Sun May 22 2016 danimo@owncloud.com
- Bump to v0.2.0
- This version fixes a json-parsing bug which made letsencrypt.sh
  incompatible with up-to-date ACME servers.
- PRIVATE_KEY config parameter has been renamed to ACCOUNT_KEY to avoid
  confusion with certificate keys
- deploy_cert hook now also has the certificates timestamp as standalone
  parameter
- Temporary files are now identifiable (template: letsencrypt.sh-XXXXXX)
- Private keys are now regenerated by default
- Added documentation to repository
- Fixed bug with uppercase names in domains.txt (script now converts everything
  to lowercase)
- mktemp no longer uses the deprecated -t parameter.
- Compatibility with "pretty" json
* Wed Apr 20 2016 danimo@owncloud.com
- Explicitly add group and license, required for SLES 11
* Wed Apr 20 2016 danimo@owncloud.com
- Add nginx integration package
- Proper dir permissions for apache package (755, not 644)
* Mon Apr 18 2016 draht@schaltsekun.de
- fix build requirement for shadow (>=openSUSE-12.3) and pwdutils
  (before 12.3).
- missing changelog for last change by danimo: do not require mod_ssl for
  suse distrbutions.
* Mon Mar 28 2016 danimo@owncloud.com
- Add alias to /.well-known/acme-challenge by default
* Sat Mar 26 2016 danimo@owncloud.com
- Add cron, do not remove letsencrypt user, adjust permissions
* Fri Mar 25 2016 danimo@owncloud.com
- Initial commit
